import { Observable } from 'rxjs';
import {ObserveCache} from "./observeCache";
import {Account, Client, Functions, Models} from 'appwrite';
import {CollectionsList} from "../components/types";

class AWService {
    private awEndpoint: string = "";
    private awProject: string = "";
    private readonly awClient: Client;
    private collectionsObserveCache: ObserveCache<CollectionsList> = new ObserveCache<CollectionsList>();

    constructor() {
        this.awClient = new Client();
    }

    public getCollections(): Observable<CollectionsList> {
        let collectionRequest: () => Promise<CollectionsList> = () => {
            let account = new Account(this.awClient);
            let functions = new Functions(this.awClient);

            return account.get().then(() => {
                return functions.createExecution("getCollections")
            }).then((executionInfo: Models.Execution) => {
                return waitForCompletion();

                async function waitForCompletion(): Promise<Models.Execution> {
                    let complete = false;
                    let exec: Models.Execution = await getExec();
                    if (exec.status !== "processing") {
                        complete = true;
                    }
                    while (!complete) {
                        exec = await delay();
                        if (exec.status !== "processing") {
                            complete = true;
                        }
                    }
                    return exec;
                }

                async function delay(): Promise<Models.Execution> {
                    return new Promise<Models.Execution>((resolve, reject) => {
                        setTimeout(() => {
                            getExec().then((exec: Models.Execution) => {
                                resolve(exec);
                            });
                        }, 2000);
                    })
                }

                function getExec(): Promise<Models.Execution> {
                    return functions.getExecution("getCollections", executionInfo.$id);
                }
            }).then((ex: Models.Execution) => {
                return JSON.parse(ex.response) as CollectionsList;
            }).catch((err) => {
                console.error(err);
                return {collections: [], total: 0};
            });
        }
        return this.collectionsObserveCache.get('collections', collectionRequest);
    }

    public init(endpoint: string, project: string) {
        if (this.awEndpoint === "") {
            this.awEndpoint = endpoint;
            this.awClient.setEndpoint(endpoint);
        }
        if (this.awProject === "") {
            this.awProject = project;
            this.awClient.setProject(project);
        }
        // otherwise we have already been initialized, and we don't need to do anything
    }
}

const awService = new AWService();

export {
    awService
    // AWService
}
