import { Observable, from } from 'rxjs';
import { get } from './fetch';
import { ObserveService } from './observeService';

interface storedItem<T> {
    observeService: ObserveService<T>
    bust: boolean
    updated: number
}

class ObserveCache<T> {
    private cacheStore: { [key: string]: storedItem<T> } = {};
    private expiration: number = 5 * 60 * 1000; // 5 minutes

    public get(key: string, endpoint: string|(()=>Promise<T>)) {
        let now = new Date().getTime();
        if (!this.cacheStore[key]) {
            function getRequest() {
                if (typeof endpoint === 'string') {
                    return from<Promise<T>>(get<T>(endpoint));
                }
                return from<Promise<T>>(endpoint());
            }
            this.cacheStore[key] = {
                observeService: new ObserveService<T>(getRequest),
                bust: false,
                updated: now,
            };
        } else if (this.cacheStore[key].bust || now - this.cacheStore[key].updated > this.expiration) {
            this.cacheStore[key].observeService.next();
            this.cacheStore[key].updated = now;
        }
        return this.cacheStore[key].observeService.value$;
    }

    public bust(key: string) {
        this.cacheStore[key].bust = true;
    }
}

export {
    ObserveCache
}
