import {merge, Observable, Subject} from 'rxjs';
import {debounceTime, shareReplay, switchMap} from 'rxjs/operators';

export class ObserveService<T> {
    private updateStream: Subject<void>;
    public value$: Observable<T>;
    // TODO: add timestamp for last updated so the cache can be smart about when to check for new values

    constructor(request: ()=>Observable<T>) {
        this.updateStream = new Subject<void>();
        const initialCurrentResponse$ = request();
        const reload$ = this.updateStream.pipe(debounceTime(1000), switchMap(() => request()));
        this.value$ = merge(initialCurrentResponse$, reload$).pipe(shareReplay(1));
    }

    public next() {
        this.updateStream.next();
    }
}
